      REAL FUNCTION rpv2 (thetas, thetav, relphi, rho0, k,
     &   bigtet, rhoc, verbos, err)
      IMPLICIT NONE
C
C***********************************************************************
C
C  Function RPV
C  Version 2.1, 09 February 2022 (Christian Lanconelli)
C
C  Version 1.1, 28 February 2001
C
C  Author: Michel M. Verstraete
C          Space Applications Institute (SAI)
C          EC Joint Research Centre, TP 440
C          I-21020 Ispra (VA), Italy
C          E-mail: Michel.Verstraete@jrc.it
C
C  MODIFIED: 11/2008 JLW: enable 4 parameter version of rpv
c
C  Purpose:
C
C  To return the value of the bidirectional reflectance factor for an
C  arbitrary geophysical medium, given the illumination and observation
C  geometry, as well as the values of three coefficients describing the
C  anisotropy of that medium, as computed with the RPV model.
C
C  Computer language:
C
C  FORTRAN-77
C
C  Argument(s):
C
      REAL         thetas, thetav, relphi, rho0, k, bigtet, rhoc
      LOGICAL      verbos
      INTEGER      err
C
C  Definitions:
C
C  thetas          Illumination zenith angle, in radians [0-89]
C  thetav          Observation zenith angle, in radians  [0-89]
C  relphi          Relative azimuth angle, in radians.
C                  The code verifies that relphi is within the
C                  range equivalent to [0, 180] degrees, where 0
C                  corresponds to forward and 180 to backward
C                  scattering (hot spot).
C  rho0            Overall reflectance level parameter, N/D.
C  k               Anisotropy shape parameter, N/D.
C                  k < 1.0 (> 1.0) corresponds to a bowl (bell) shape.
C  bigtet          Asymmetry parameter, N/D.
C                  bigtet < 0.0 (> 0.0) corresponds to a
C                  predominantly backward (forward) scattering.
C  rhoc            hotspot function parameter (1.0 = no hotspot), N/D.
C  verbos          Flag to print out input, intermediary and
C                  output values on standard output for
C                  diagnostic purposes. Please note that this setting
C                  has no effect if an error is detected (see below).
C  err             Error code. This variable is set to a non-zero value
C                  whenever an erroneous input or a computational error
C                  is detected.
C                  err = 0 under normal conditions.
C                  err = 10 and rpv = -1.0 for invalid thetas.
C                  err = 20 and rpv = -1.0 for invalid thetav.
C                  err = 30 and rpv = -1.0 for invalid relphi.
C                  err = 100 and rpv = -1.0 if the denominator of the
C                  Henyey and Greenstein phase function vanishes.
C
C  Copyright (C) 2001 Michel M. Verstraete:
C
C     This code is made available publicly for scientific purposes.
C     Nevertheless, the copyright remains with its author, and any
C     use of this code must be accompanied by appropriate and usual
C     acknowledgments, by referring to the origin of the source code
C     and to the publications indicated below. If your students or
C     colleagues also need to access this code for research purposes,
C     please refer them to one of the official source sites (e.g.,
C     http://www.enamors.org/), where they can be sure to download
C     the ultimate version.
C     Use of this code (RPV) in commercial applications is strictly
C     forbidden without the written approval of the author. Even with
C     such an authorization to use the code, you may not distribute
C     or sell it to any other commercial or business partners under
C     any circumstances.
C
C  References:
C
C  1. Rahman, H., Verstraete M. M. and Pinty B. (1993) `Coupled
C     Surface-Atmosphere Reflectance (CSAR) Model. Part 1: Model
C     Description and Inversion on Synthetic Data', Journal of
C     Geophysical Research, 98, 20,779-20,789.
C  2. Rahman, H., Pinty B. and Verstraete M. M. (1993) `Coupled 
C     Surface-Atmosphere Reflectance (CSAR) Model. Part 2: 
C     Semiempirical Surface Model Usable With NOAA Advanced Very High
C     Resolution Radiometer Data', Journal of Geophysical Research, 98,
C     20,791-20,801.
C
C  Copies of these two papers can be obtained from Michel M. Verstraete
C  at the address above.
C
C  History of releases:
C
C  Date        Version  Notes
C  ====        =======  =====
C  1993-07-01  1.0      Initial publication of the model.
C  2001-02-28  1.1      Certification, documentation, and public
C                       release of this code.
C  2020-02-05  2.0      Angles as you eat ... &hotspot in dPHI==0
C  2022-02-09  2.1      implicit none (some minor bugs) 
C
C***********************************************************************
C
C  Declaration of local variables:
C
      REAL coss, cosv, sins, sinv, cosp, tans, tanv
      REAL bgthsq, csmllg, bigg, expon, denom, f1, f2, f3
      REAL r80, r100, r180
      INTRINSIC ABS, COS, SQRT
      DATA r80 /1.396263402/, r100 /1.745329252/, r180 /3.141592654/
C
C  Initialize the error code and check input values:
C
      err = 0
C      IF ((thetas .LT. r100) .OR. (thetas .GT. r180)) THEN
      IF ((thetas .LT. 0.) .OR. (thetas .GT. 89.)) THEN  ! C.Lanconelli
         err = 10
         rpv2 = -1.0
         RETURN
      ENDIF
C      IF ((thetav .LT. 0.0) .OR. (thetav .GT. r80)) THEN
      IF ((thetav .LT. 0.0) .OR. (thetav .GT. 89.)) THEN ! C.Lanconelli
         err = 20
         rpv2 = -1.0
         RETURN
      ENDIF
      
C Any Relative angle can be managed ...
C      IF ((relphi .LT. 0.0) .OR. (relphi .GT. r180)) THEN
C         err = 30
C         rpv = -1.0
C         RETURN
C      ENDIF

C
C  Compute the trigonometric functions:
C
C      coss = ABS (COS (thetas))
      coss = COS (thetas)  !C.Lanconelli 2020-02 (All zenith [0-89])
      cosv = COS (thetav)
      sins = SQRT (1.0 - coss * coss)
      sinv = SQRT (1.0 - cosv * cosv)
C      cosp = -COS (relphi)  ! Was (ver<2.0) 
      cosp = COS (relphi)    ! C.Lanconelli 2020-02 (Forward relphi=180, Backward relphi=0)
      tans = sins / coss
      tanv = sinv / cosv
      csmllg = coss * cosv + sins * sinv * cosp
      bigg = SQRT (tans * tans + tanv * tanv -
     &   2.0 * tans * tanv * cosp)
      bgthsq = bigtet * bigtet
C
C  Compute the Minnaert-like factor:
C
      expon = k - 1.0
      IF (expon .NE. 0.0) THEN
         f1 = ((coss * cosv) ** expon) *
     &      ((coss + cosv) ** expon)
      ELSE
         f1 = 1.0
      ENDIF
C
C  Compute the asymmetry function (Henyey and Greenstein):
C
      denom = (1.0 + bgthsq + 2.0 * bigtet * csmllg) ** (1.5)
      IF (denom .NE. 0.0) THEN
         f2 = (1.0 - bgthsq) / denom
      ELSE
         err = 100
         rpv2 = -1.0
         RETURN
      ENDIF
C
C  Compute the hotspot function
C
      f3 = (1.0 + ((1 - rhoc) / (1.0 + bigg)))
C
C  Compute the overall RPV output
C     
      rpv2 = rho0 * f1 * f2 * f3
C
C  Remark:
C
C  The following IF-ENDIF block is useful for diagnostic purposes
C  only, and should not be used under operational conditions.
C  Whenever run-time efficiency is a prime consideration,
C  1. remove the 'verbos' variable from the argument list,
C  2. delete the associated definition, and
C  3. delete the following IF-ENDIF block.
C
C  If the verbos variable is set, print selected input, intermediary
C  and output variables on standard output:
C
      IF (verbos) THEN
         WRITE (6, '(3(A10, F10.6))') ' thetas = ', thetas,
     &      ' thetav = ', thetav, ' relphi = ', relphi
         WRITE (6, '(3(A10, F10.6))') ' rho0 = ', rho0,
     &      ' k = ', k, ' bigtet = ', bigtet
         WRITE (6, '(2(A10, F10.6))') ' csmllg = ', csmllg,
     &      ' bigg = ', bigg
         WRITE (6, '(3(A10, F10.6))') ' f1 = ', f1, ' f2 = ', f2
         WRITE (6, '(A10, I10, 20X, A10, F10.6)') ' err = ', err,
     &      ' rpv = ', rpv2
         WRITE (6, '(A1)') ' '
      ENDIF
C
      RETURN
      END
